var request = require("request");

exports.get = APIRequest.bind(this, "GET");
exports.post = APIRequest.bind(this, "POST");
exports.put = APIRequest.bind(this, "PUT");
exports.delete = APIRequest.bind(this, "DELETE");

function APIRequest(method, req, callback) {
  const uri = getRequestURI(req);
  const options = {
    json: true,
    headers: {
      "Content-Type": "application/json"
    }
  };

  if (req.body && Object.keys(req.body).length) {
    options.body = req.body;
  }

  if (req.session.user) {
    options.headers["Authorization"] = "bearer " + req.session.user.access_token;
  }

  request(Object.assign(options, {uri, method, timeout: 5000}), function (error, response, body) {
    "use strict";

    // handle case when API is not available
    if (error) {
      let statusCode;

      switch (error.code) {
        case "ECONNREFUSED":
          statusCode = 503;
          break;
        case "ETIMEDOUT":
          statusCode = 506;
          break;
        default:
          statusCode = 500;
      }

      response = {statusCode};
    }

    callback(error, response, body);
  });
}

function getRequestURI(req) {
  return req.uri || req.environment.hosts.api + (req.urn || req.originalUrl);
} 